#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <TimerOne.h>

const unsigned long interval = 10000;  // Intervalul de timp în milisecunde (10 secunde)
unsigned long previousMillis = 0;     // Variabila pentru stocarea timpului anterior


#define SCREEN_WIDTH 128 //  width OLED in pixeli
#define SCREEN_HEIGHT 64 //  height OLED in pixeli

#define OLED_DC     8
#define OLED_CS     10
#define OLED_RESET  9
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT,
  &SPI, OLED_DC, OLED_RESET, OLED_CS);

int carWidth = 68;
int carHeight = 48;
const unsigned char epd_bitmap_car [] PROGMEM = {
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xef, 0xff, 0xf0, 0xff, 0xfe, 0x38, 0x00, 0x00, 0x01, 0xc7, 
  0xff, 0xf0, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xf0, 0xff, 0xc0, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x3f, 0xf0, 0xff, 0x87, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x3f, 0xf0, 0xff, 0x8f, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0x1f, 0xf0, 0xff, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x8f, 0xf0, 0xfe, 
  0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x87, 0xf0, 0xfe, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc7, 
  0xf0, 0xfc, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xf0, 0xf8, 0x7f, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xe1, 0xf0, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf0, 0xf0, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf0, 0xe0, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 
  0xdf, 0xff, 0xff, 0xff, 0x9f, 0xff, 0xff, 0xff, 0xb0, 0xdf, 0xff, 0xff, 0xe0, 0xb0, 0x7f, 0xff, 
  0xff, 0xb0, 0x03, 0xff, 0xf8, 0x00, 0x00, 0x03, 0xff, 0xfc, 0x10, 0x07, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xfc, 0x00, 0x07, 0xff, 0xff, 0xe0, 0x00, 0x7f, 0xff, 0xfe, 0x00, 0x87, 0xff, 0xff, 
  0xe0, 0x00, 0x7f, 0xff, 0xfe, 0x10, 0x87, 0xff, 0xff, 0xe0, 0x00, 0x7f, 0xff, 0xfe, 0x10, 0x07, 
  0xff, 0xff, 0xe0, 0x00, 0x7f, 0xff, 0xfe, 0x10, 0x07, 0xff, 0xff, 0xe0, 0x00, 0x7f, 0xff, 0xfe, 
  0x00, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xfe, 0x00, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x07, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xfe, 0x00, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 
  0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x10, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 
  0xf0, 0x00, 0x10, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x10, 0x80, 0x01, 0xff, 0xff, 
  0xff, 0xff, 0xf0, 0x00, 0x10, 0x80, 0x01, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x10, 0x80, 0x01, 
  0xff, 0xff, 0xff, 0xff, 0xf0, 0x00, 0x10, 0x80, 0x01, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x10
};

int checkLines[3][2] = {
  {1, 1},
  {1, 1},
  {1, 1}
  };
  
const int pingPinR = 7; // Tr Pin 
const int echoPinR = 6; // Ec Pin 
const int pingPinL = 5; 
const int echoPinL = 4;

const int RedL = A2;
const int GreenL = A1;
const int BlueL = A0;

const int RedR = A5;
const int GreenR = A4;
const int BlueR = A3;

const int Buzzer = 3;
const int buttonPin = 2;
int BuzzerAct = HIGH;

long distanceL = -1;//0
long distanceR = -1;//1

const int maxDist = 20;
const int mediumDist = 10;


void setup() {
    pinMode(buttonPin, INPUT_PULLUP);
   if(!display.begin(SSD1306_SWITCHCAPVCC)) {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
   }
   pinMode(RedL, OUTPUT);
   pinMode(GreenL, OUTPUT);
   pinMode(BlueL, OUTPUT);
   pinMode(RedR, OUTPUT);
   pinMode(GreenR, OUTPUT);
   pinMode(BlueR, OUTPUT);

   attachInterrupt(digitalPinToInterrupt(buttonPin), accidentISR, FALLING);
   Serial.begin(9600); // Starting Serial Terminal

   Timer1.initialize(interval);  // Inițializează Timer1 cu intervalul dorit
   Timer1.attachInterrupt(timerISR);  // Atașează funcția timerISR ca întrerupere pentru Timer1

}
void loop() {
   distanceL = readDistanceSenzorData(pingPinL, echoPinL);
   distanceR = readDistanceSenzorData(pingPinR, echoPinR);
   //Serial.println(distanceL);
   colorLedDistance(distanceL, 0);
   colorLedDistance(distanceR, 1);
   //updatePrevBut();
   buzzerActivation();
   showDisplay();
   delay(20);
   
}
void accidentISR() {
  Serial.println("Ați făcut accident!");
}
void timerISR() {
  unsigned long currentMillis = millis();  // Obține timpul curent în milisecunde

  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;  // Actualizează timpul anterior

    Serial.println("Faceti pauza de cafea");
  }
}
void showDisplay(){
  display.clearDisplay();
  display.drawBitmap((display.width() - carWidth)/2,0,epd_bitmap_car,carWidth, carHeight, SSD1306_INVERSE);
  drawCheckLines();
  updateCheckLines();
}

void drawCheckLines(){
   for(int i = 0;i < 3;i++){
      for(int j = 0;j < 2;j++){
        if(checkLines[i][j] == 1){
          display.fillRect((display.width() - carWidth)/2 + j *36 , 2 + carHeight + i * 5, 32, 4, SSD1306_INVERSE);
        }
      }
   }
   display.display();
   delay(1);
}

void updateCheckLines(){
  for(int i = 0;i < 3;i++){
      for(int j = 0;j < 2;j++){
        checkLines[i][j] = 1;
      }
    }
  if(distanceL < maxDist && distanceL >= mediumDist){
    checkLines[0][1] = 0;
  }
  if(distanceL < mediumDist){
    checkLines[0][1] = 0;
    checkLines[1][1] = 0;
  }

  if(distanceR < maxDist && distanceR >= mediumDist){
    checkLines[0][0] = 0;
  }
  if(distanceR < mediumDist){
    checkLines[0][0] = 0;
    checkLines[1][0] = 0;
  }
}


void colorLedDistance(long distance,int L_R){
  if(L_R == 0){
    if(distance >= maxDist){
      lightRGBLed(RedL, GreenL, BlueL, 0, 255, 0);
    } else if(distance >= mediumDist){
      lightRGBLed(RedL, GreenL, BlueL, 255, 255, 0);
    } else {
      lightRGBLed(RedL, GreenL, BlueL, 255, 0, 0);
    }
  } else {
     if(distance >= maxDist){
      lightRGBLed(RedR, GreenR, BlueR, 0, 255, 0);
    } else if(distance >= mediumDist){
      lightRGBLed(RedR, GreenR, BlueR, 255, 255, 0);
    } else {
      lightRGBLed(RedR, GreenR, BlueR, 255, 0, 0);
    }
  }
}

void buzzerActivation(){
  if((distanceL < mediumDist || distanceR < mediumDist) && BuzzerAct == HIGH){
    //medium zone
    tone(Buzzer, 2000);
  } else if(((distanceL >= mediumDist && distanceL <= maxDist) || (distanceR >= mediumDist && distanceR <= maxDist)) && BuzzerAct == HIGH ){
    tone(Buzzer, 1000);
  } else {
    noTone(Buzzer);
  }
}


void lightRGBLed(int RPin, int GPin, int BPin, int RValue, int GValue, int BValue){
  analogWrite(RPin, RValue);
  analogWrite(GPin, GValue);
  analogWrite(BPin, BValue);
}

long readDistanceSenzorData(int ping, int echo){
   long duration, inches, cm, distance;
   pinMode(ping, OUTPUT);
   digitalWrite(ping, LOW);
   delayMicroseconds(2);
   digitalWrite(ping, HIGH);
   delayMicroseconds(10);
   digitalWrite(ping, LOW);
   pinMode(echo, INPUT);
   duration = pulseIn(echo, HIGH);
   distance = microsecondsToCentimeters(duration);
   return distance;
}

long microsecondsToCentimeters(long microseconds) {
   return microseconds / 29 / 2;
}
